@@ -29,28 +29,28 @@ from .pyqt import PyQt
 
 # The Qt meta-data for this package.
 _QT_METADATA = {
-    #'QAxContainer':
-    #    VersionedMetadata(dll=False),
+    'QAxContainer':
+        VersionedMetadata(dll=False),
 
-    #'QtBluetooth':
-    #    VersionedMetadata(
-    #        lib_deps={
-    #            'linux': ('QtConcurrent', ),
-    #            'macos': ('QtConcurrent', )},
-    #        qml=True),
+    'QtBluetooth':
+        VersionedMetadata(version=(6, 2, 0), qml=True),
 
-    'QtCore':
+    'QtCore': (
+        VersionedMetadata(version=(6, 2, 0),
+            other_lib_deps={
+                'linux': ('libicui18n.so.56', 'libicuuc.so.56',
+                          'libicudata.so.56')},
+            translations=('qt_', 'qt_help', 'qtbase', 'qtdeclarative', 
+                'qtlocation', 'qtmultimedia', 'qtquickcontrols2',
+                'qtserialport', 'qtwebsockets'),
+            excluded_plugins=('designer', 'qmltooling')),
         VersionedMetadata(
             other_lib_deps={
                 'linux': ('libicui18n.so.56', 'libicuuc.so.56',
                           'libicudata.so.56')},
-            translations=('qt_help', 'qtbase', 'qtdeclarative', 
-                'qtquickcontrols2', 'qt_'),
-            #translations=('qt_help', 'qtbase', 'qtconnectivity',
-            #    'qtdeclarative', 'qtlocation', 'qtmultimedia',
-            #    'qtquickcontrols2', 'qtserialport', 'qtwebsockets',
-            #    'qtxmlpatterns', 'qt_'),
-            excluded_plugins=('designer', 'qmltooling')),
+            translations=('qt_', 'qt_help', 'qtbase', 'qtdeclarative', 
+                'qtquickcontrols2'),
+            excluded_plugins=('designer', 'qmltooling'))),
 
     'QtDBus':
         VersionedMetadata(),
@@ -58,32 +58,34 @@ _QT_METADATA = {
     'QtDesigner':
         VersionedMetadata(),
 
-    'QtGui':
-        VersionedMetadata(lib_deps={'linux': ('QtWaylandClient', 'QtXcbQpa')},
+    'QtGui': (
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'linux': ('QtWaylandClient', 'QtXcbQpa')},
                 other_lib_deps={
                         'win': ('d3dcompiler_47.dll', 'opengl32sw.dll')}),
+        VersionedMetadata(lib_deps={'linux': ('QtWaylandClient', 'QtXcbQpa')},
+                other_lib_deps={
+                        'win': ('d3dcompiler_47.dll', 'opengl32sw.dll')})),
 
     'QtHelp':
         VersionedMetadata(),
 
     #'QtLocation':
-    #    VersionedMetadata(
-    #            lib_deps={'': ('QtPositioningQuick', )},
-    #            qml=True),
+    #    VersionedMetadata(qml=True),
 
-    #'QtMultimedia':
-    #    VersionedMetadata(
-    #            lib_deps={'linux': ('QtMultimediaGstTools', )},
-    #            qml=True, qml_names=('QtAudioEngine', 'QtMultimedia')),
+    'QtMultimedia':
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'': ('QtMultimediaQuick', )},
+                qml=True),
 
-    #'QtMultimediaWidgets':
-    #    VersionedMetadata(),
+    'QtMultimediaWidgets':
+        VersionedMetadata(version=(6, 2, 0)),
 
     'QtNetwork':
         VersionedMetadata(),
 
-    #'QtNfc':
-    #    VersionedMetadata(qml=True),
+    'QtNfc':
+        VersionedMetadata(version=(6, 2, 0), qml=True),
 
     'QtOpenGL':
         VersionedMetadata(),
@@ -91,8 +93,10 @@ _QT_METADATA = {
     'QtOpenGLWidgets':
         VersionedMetadata(),
 
-    #'QtPositioning':
-    #    VersionedMetadata(qml=True),
+    'QtPositioning':
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'': ('QtPositioningQuick', )},
+                qml=True),
 
     'QtPrintSupport':
         VersionedMetadata(),
@@ -102,19 +106,29 @@ _QT_METADATA = {
                 lib_deps={'': ('QtQmlModels', 'QtQmlWorkerScript')},
                 qml=True),
 
-    'QtQuick':
+    'QtQuick': (
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'': ('QtQuickControls2', 'QtQuickControls2Impl',
+                        'QtQuickDialogs2', 'QtQuickDialogs2QuickImpl',
+                        'QtQuickDialogs2Utils', 'QtQuickLayouts',
+                        'QtQuickParticles', 'QtQuickShapes',
+                        'QtQuickTemplates2', 'QtQuickTest',
+                        'QtQuickTimeline')},
+                qml=True),
         VersionedMetadata(
                 lib_deps={'': ('QtQuickControls2', 'QtQuickControls2Impl',
                         'QtQuickLayouts', 'QtQuickParticles', 'QtQuickShapes',
                         'QtQuickTemplates2', 'QtQuickTest')},
-                qml=True),
+                qml=True)),
 
     'QtQuick3D': (
         VersionedMetadata(version=(6, 1, 0),
                 lib_deps={
-                        '': ('QtQuick3DAssetImport', 'QtQuick3DIblBaker',
-                        'QtQuick3DParticles', 'QtQuick3DRuntimeRender',
-                        'QtQuick3DUtils', 'QtShaderTools')},
+                        '': ('QtQuick3DAssetImport', 'QtQuick3DAssetUtils',
+                        'QtQuick3DEffects', 'QtQuick3DHelpers',
+                        'QtQuick3DIblBaker', 'QtQuick3DParticles',
+                        'QtQuick3DRuntimeRender', 'QtQuick3DUtils',
+                        'QtShaderTools')},
                 qml=True),
         VersionedMetadata(
                 lib_deps={
@@ -125,14 +139,18 @@ _QT_METADATA = {
     'QtQuickWidgets':
         VersionedMetadata(),
 
-    #'QtRemoteObjects':
-    #    VersionedMetadata(qml=True),
+    'QtRemoteObjects':
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'': ('QtRemoteObjectsQml', )},
+                qml=True),
 
-    #'QtSensors':
-    #    VersionedMetadata(qml=True),
+    'QtSensors':
+        VersionedMetadata(version=(6, 2, 0),
+                lib_deps={'': ('QtSensorsQuick', )},
+                qml=True),
 
-    #'QtSerialPort':
-    #    VersionedMetadata(),
+    'QtSerialPort':
+        VersionedMetadata(version=(6, 2, 0)),
 
     'QtSql':
         VersionedMetadata(),
@@ -149,11 +167,11 @@ _QT_METADATA = {
     #'QtTextToSpeech':
     #    VersionedMetadata(),
 
-    #'QtWebChannel':
-    #    VersionedMetadata(qml=True),
+    'QtWebChannel':
+        VersionedMetadata(version=(6, 2, 0), qml=True),
 
-    #'QtWebSockets':
-    #    VersionedMetadata(qml=True),
+    'QtWebSockets':
+        VersionedMetadata(version=(6, 2, 0), qml=True),
 
     'QtWidgets':
         VersionedMetadata(),
